l("moo")
;l("mo\"o")
;l('mo\"oo')
;l('mo\'ooo')
;l('mo\oooo')
;l('mo\\oooo')
;l('mo"ooooo')
;l("mo'ooooo")
;ld("dummy","mo'ooooo")
;l(' Sie müssen eine eigene Domain eingeben. "%s" ist nicht zulässig. ', l('ihr-name'))
;l("Sie müssen eine eigene Domain eingeben.".' "%s" ist nicht zulässig!')
;l("mo'o
   oooo")
;l(
   "mo'o
   oooo"
   )
;l
(
   "mo'o" .
   "oooo"
   )
